/*******************************************************************************
*
*	TITLE		gyro.c 
*
*	VERSION:	0.3 (Beta)                           
*
*	DATE:		21-Jan-2005
*
*	AUTHOR:		R. Kevin Watson
*				kevinw@jpl.nasa.gov
*
*	COMMENTS:
*
********************************************************************************
*
*	CHANGE LOG:
*
*	DATE         REV  DESCRIPTION
*	-----------  ---  ----------------------------------------------------------
*	21-Nov-2004  0.1  RKW - Original code.
*	12-Jan-2005  0.2  RKW - Altered Get_Gyro_Rate() and Get_Gyro_Angle() to use
*	                  long integers for internal calculations, allowing larger
*	                  numerators and denominators in the GYRO_RATE_SCALE_FACTOR
*	                  and GYRO_ANGLE_SCALE_FACTOR #defines.
*	12-Jan-2005  0.2  RKW - GYRO_RATE_SCALE_FACTOR and GYRO_ANGLE_SCALE_FACTOR
*	                  #defines added for Analog Devices' ADXRS150 and ADXRS300 
*	                  gyros.
*	16-Jan-2005  0.3  RKW - Using preprocessor directives, added the ability 
*	                  to select the gyro type, angular units, sample rate and
*	                  number of averaged samples per update.
*	21-Jan-2005  0.3  RKW - Added scaling factors for the BEI GyroChip.
*
*******************************************************************************/

#include "gyro.h"
#include "ifi_aliases.h"
#include "ifi_default.h"

volatile int gyro_bias;
volatile int gyro_rate;
volatile long gyro_angle;
unsigned char calc_gyro_bias = 0;

/*******************************************************************************
*
*	FUNCTION:		Initialize_Gyro()
*
*	PURPOSE:		Initializes the gyro code
*
*	CALLED FROM:	user_routines.c/User_Initialization()
*
*	PARAMETERS:		None
*
*	RETURNS:		Nothing
*
*	COMMENTS:		Place "#include "gyro.h" in the includes section
*					of user_routines.c then call Initialize_Gyro() in
*					user_routines.c/User_Initialization().
*
*******************************************************************************/
void Initialize_Gyro(void)
{
	// reset the heading angle
	Reset_Gyro_Angle();

	// force a gyro bias calculation
	Calc_Gyro_Bias();

	// initialize the analog to digital converter
	Initialize_ADC();

	// initialize and start timer 2
	Initialize_Timer_2();
}

/*******************************************************************************
*
*	FUNCTION:		Get_Gyro_Rate()
*
*	PURPOSE:		Returns the current heading angular rate of change
*
*	CALLED FROM:
*
*	PARAMETERS:		None
*
*	RETURNS:		Nothing
*
*	COMMENTS:
*
*******************************************************************************/
int Get_Gyro_Rate(void)
{
	int temp_gyro_rate;

	// Since we're about to use the gyro_rate variable, which can 
	// also be modified in the timer 2 interrupt service routine,
	// let's briefly disable the timer 2 interrupt to make sure 
	// that gyro_rate doesn't get altered while we're using it.
	PIE1bits.TMR2IE = 0;

	// Now we can get a local copy of gyro_rate without fear that
	// we'll get corrupted data.
	temp_gyro_rate = gyro_rate;

	// We have a local copy of gyro_rate, so turn the timer 2 
	// interrupt back on.
	PIE1bits.TMR2IE = 1;

	// Return the calculated gyro rate to the caller.
	return((int)((long)temp_gyro_rate * (long)GYRO_RATE_SCALE_FACTOR / 
						(long)GYRO_SAMPLES_PER_UPDATE * (long)GYRO_CAL_FACTOR));
}

/*******************************************************************************
*
*	FUNCTION:		Get_Gyro_Angle()
*
*	PURPOSE:		Returns the current heading angle
*
*	CALLED FROM:
*
*	PARAMETERS:		None
*
*	RETURNS:		Nothing
*
*	COMMENTS:
*
*******************************************************************************/
long Get_Gyro_Angle(void)
{
	long temp_gyro_angle;

	// Since we're about to use the gyro_angle variable, which can 
	// also be modified in the timer 2 interrupt service routine,
	// let's briefly disable the timer 2 interrupt to make sure 
	// that gyro_angle doesn't get altered while we're using it.
	PIE1bits.TMR2IE = 0;

	// Now we can get a local copy of gyro_angle without fear that
	// we'll get corrupted data.
	temp_gyro_angle = gyro_angle;

	// We have a local copy of gyro_angle, so turn the timer 2 
	// interrupt back on.
	PIE1bits.TMR2IE = 1;

	// Return the calculated gyro angle to the caller.
	return(temp_gyro_angle * (long)GYRO_RATE_SCALE_FACTOR / 
						(long)GYRO_SAMPLE_RATE * (long)GYRO_CAL_FACTOR);
}

/*******************************************************************************
*
*	FUNCTION:		Calc_Gyro_Bias()
*
*	PURPOSE:		Calculates the gyro bias and updates the internal
*					gyro_bias variable.
*
*	CALLED FROM:
*
*	PARAMETERS:		None
*
*	RETURNS:		Nothing
*
*	COMMENTS:		For best results, only call this function when your 
*					robot is still and free of vibration (e.g., the air 
*					compressor is off).
*
*******************************************************************************/
void Calc_Gyro_Bias(void)
{
	// inform the timer 2 interrupt service routine 
	// that a new gyro bias needs to be calculated
	calc_gyro_bias = 1;
}

/*******************************************************************************
*
*	FUNCTION:		Get_Gyro_Bias()
*
*	PURPOSE:		Returns the current gyro bias.		
*
*	CALLED FROM:
*
*	PARAMETERS:		None
*
*	RETURNS:		Nothing
*
*	COMMENTS:		For extra precision, this software internally maintains a 
*					gyro bias value that is the sum of GYRO_SAMPLES_PER_UPDATE 
*					samples of the gyro's analog output.
*
*******************************************************************************/
int Get_Gyro_Bias(void)
{
	int temp_gyro_bias;

	// Since we're about to use the gyro_bias variable, which can 
	// also be modified in the timer 2 interrupt service routine,
	// let's briefly disable the timer 2 interrupt to make sure 
	// that gyro_bias doesn't get altered while we're using it.
	PIE1bits.TMR2IE = 0;

	// Now we can get a local copy of gyro_bias without fear that
	// we'll get corrupted data.
	temp_gyro_bias = gyro_bias;

	// We have a local copy of gyro_bias, so turn the timer 2 
	// interrupt back on.
	PIE1bits.TMR2IE = 1;

	// return the gyro bias to the caller
	return(temp_gyro_bias);
}

/*******************************************************************************
*
*	FUNCTION:		Set_Gyro_Bias()
*
*	PURPOSE:		Manually sets the gyro bias.
*
*	CALLED FROM:
*
*	PARAMETERS:		None
*
*	RETURNS:		Nothing
*
*	COMMENTS:		For extra precision, this software internally maintains a 
*					gyro bias value that is the sum of GYRO_SAMPLES_PER_UPDATE 
*					samples of the gyro's analog output.
*
*******************************************************************************/
void Set_Gyro_Bias(int new_gyro_bias)
{
	// Since we're about to use the gyro_bias variable, which can 
	// also be modified in the timer 2 interrupt service routine,
	// let's briefly disable the timer 2 interrupt to make sure 
	// that gyro_bias doesn't get altered while we're updating it.
	PIE1bits.TMR2IE = 0;

	// Now we can update gyro_bias without fear of corruption.
	gyro_bias = new_gyro_bias;

	// We're done updating the gyro_bias variable, so turn the 
	// timer 2 interrupt back on.
	PIE1bits.TMR2IE = 1;
}

/*******************************************************************************
*
*	FUNCTION:		Reset_Gyro_Angle()
*
*	PURPOSE:		Resets the heading angle to zero.
*
*	CALLED FROM:
*
*	PARAMETERS:		None
*
*	RETURNS:		Nothing
*
*	COMMENTS:
*
*******************************************************************************/
void Reset_Gyro_Angle(void)
{
	// Since we're about to use the gyro_angle variable, which can 
	// also be modified in the timer 2 interrupt service routine,
	// let's briefly disable the timer 2 interrupt to make sure 
	// that gyro_angle doesn't get altered while we're updating it.
	PIE1bits.TMR2IE = 0;

	// zero out gyro_angle
	gyro_angle = 0L;

	// We're done updating the gyro_angle variable, so turn the 
	// timer 2 interrupt back on.
	PIE1bits.TMR2IE = 1;
}

/*******************************************************************************
*
*	FUNCTION:		Disable_Gyro()
*
*	PURPOSE:		Disables the gyro software.
*
*	CALLED FROM:
*
*	PARAMETERS:		None
*
*	RETURNS:		Nothing
*
*	COMMENTS:		This function should be called when the gyro functionality 
*					is no longer needed, thus reclaiming the time spent 
*					servicing the timer 2 interrupt.
*
*******************************************************************************/
void Disable_Gyro(void)
{
	// disable the timer 2 interrupt
	PIE1bits.TMR2IE = 0;

	// disable timer 2
	T2CONbits.TMR2ON = 1;
}

/*******************************************************************************
*
*	FUNCTION:		Initialize_ADC()
*
*	PURPOSE:		Initializes the Analog to Digital Conversion (ADC) hardware.
*
*	CALLED FROM:	gyro.c/Initialize_Gyro()
*
*	PARAMETERS:		None
*
*	RETURNS:		Nothing
*
*	COMMENTS:
*
*******************************************************************************/
void Initialize_ADC(void)  
{
	// select ADC channel 0 / RC analog input 1
	ADCON0bits.CHS0 = 0;
	ADCON0bits.CHS1 = 0;
	ADCON0bits.CHS2 = 0;
	ADCON0bits.CHS3 = 0;

	// enable the ADC hardware
	ADCON0bits.ADON = 1;

	// select Vdd/Vss as the reference voltages
	ADCON1bits.VCFG0 = 0;
	ADCON1bits.VCFG1 = 0;

	#ifdef _FRC_BOARD
	// If this is being built for the FRC-RC, enable all sixteen analog inputs 
	// This shouldn't be changed because the FRC-RC has sixteen dedicated analog
	// inputs.
	ADCON1bits.PCFG0 = 0;
	ADCON1bits.PCFG1 = 0;
	ADCON1bits.PCFG2 = 0;
	ADCON1bits.PCFG3 = 0; 

	#else
	// If this is being built for the EDU-RC, enable the default two analog
	// inputs. This must match the default value used in User_Initialization()/
	// Set_Number_of_Analog_Channels(). The ADCON1 register is documented
	// in the PIC18F8520 data sheet available at http://www.microchip.com or
	// the author's website at http://kevin.org/frc.
	ADCON1bits.PCFG0 = 1;
	ADCON1bits.PCFG1 = 0;
	ADCON1bits.PCFG2 = 1;
	ADCON1bits.PCFG3 = 1;
	#endif

	// ADC result is right justified
	ADCON2bits.ADFM = 1;

	// Select Fosc/64 as the ADC Conversion Clock
	ADCON2bits.ADCS0 = 0;
	ADCON2bits.ADCS1 = 1;
	ADCON2bits.ADCS2 = 1;
}

/*******************************************************************************
*
*	FUNCTION:		Initialize_Timer_2()
*
*	PURPOSE:		Initializes the timer 2 hardware.
*
*	CALLED FROM:	gyro.c/Initialize_Gyro()
*
*	PARAMETERS:		None
*
*	RETURNS:		Nothing
*
*	COMMENTS:		Make sure that a gyro sample rate is selected in gyro.h
*
*******************************************************************************/
void Initialize_Timer_2(void)  
{
	//
	// use these parameters for a 200Hz gyro sample rate
	//
	#ifdef GYRO_SAMPLE_RATE_200HZ
	// use a 1:16 prescaler
	T2CONbits.T2CKPS0 = 0;
	T2CONbits.T2CKPS1 = 1;

	PR2	= 221; // 223.21 - 2 is ideal

	// use a 1:14 postscaler
	T2CONbits.T2OUTPS0 = 1;
	T2CONbits.T2OUTPS1 = 0;
	T2CONbits.T2OUTPS2 = 1;
	T2CONbits.T2OUTPS3 = 1;
	#endif

	//
	// use these parameters for a 400Hz gyro sample rate
	//
	#ifdef GYRO_SAMPLE_RATE_400HZ
	// use a 1:16 prescaler
	T2CONbits.T2CKPS0 = 0;
	T2CONbits.T2CKPS1 = 1;

	PR2	= 140; // 142.05 - 2 is ideal

	// use a 1:11 postscaler
	T2CONbits.T2OUTPS0 = 0;
	T2CONbits.T2OUTPS1 = 1;
	T2CONbits.T2OUTPS2 = 0;
	T2CONbits.T2OUTPS3 = 1;
	#endif

	//
	// use these parameters for a 800Hz gyro sample rate
	//
	#ifdef GYRO_SAMPLE_RATE_800HZ
	// use a 1:16 prescaler
	T2CONbits.T2CKPS0 = 0;
	T2CONbits.T2CKPS1 = 1;

	PR2	= 69; // 71.02 - 2 is ideal

	// use a 1:11 postscaler
	T2CONbits.T2OUTPS0 = 0;
	T2CONbits.T2OUTPS1 = 1;
	T2CONbits.T2OUTPS2 = 0;
	T2CONbits.T2OUTPS3 = 1;
	#endif

	//
	// use these parameters for a 1600Hz gyro sample rate
	//
	#ifdef GYRO_SAMPLE_RATE_1600HZ
	// use a 1:4 prescaler
	T2CONbits.T2CKPS0 = 1;
	T2CONbits.T2CKPS1 = 0;

	PR2	= 140; //142.05 - 2 is ideal

	// use a 1:11 postscaler
	T2CONbits.T2OUTPS0 = 0;
	T2CONbits.T2OUTPS1 = 1;
	T2CONbits.T2OUTPS2 = 0;
	T2CONbits.T2OUTPS3 = 1;
	#endif

	// make sure the timer 2 register starts at zero
	TMR2 = 0x00;

	// timer 2 interrupt is low priority
	IPR1bits.TMR2IP = 0;

	// to prevent a spurious interrupt, make sure the interrupt flag is reset
	PIR1bits.TMR2IF = 0;

	// enable the timer 2 interrupt
	PIE1bits.TMR2IE = 1;

	// enable timer 2
	T2CONbits.TMR2ON = 1;
}

/*******************************************************************************
*
*	FUNCTION:		Timer_2_Int_Handler()
*
*	PURPOSE:		Timer 2 interrupt service routine
*
*	CALLED FROM:	user_routines_fast.c/InterruptHandlerLow()
*
*	PARAMETERS:		None
*
*	RETURNS:		Nothing
*
*	COMMENTS:
*
*******************************************************************************/
void Timer_2_Int_Handler(void)
{
	unsigned int adc_result;
	static unsigned int i = 0;
	static unsigned int accum = 0;
	static unsigned char old_calc_gyro_bias = 0;

	// if we're to calculate a new gyro bias, start with a fresh sample set.
	if(calc_gyro_bias == 1 && old_calc_gyro_bias == 0)
	{
		i = 0;
		accum = 0;
	}

	// save a copy of the track_bias variable so we'll know if 
	// it has changed the next time this function is called
	old_calc_gyro_bias = calc_gyro_bias;

	// get the latest ADC conversion
	adc_result = ADRESH;
	adc_result <<= 8;
	adc_result += ADRESL;

	// add the ADC data to the accumulator
	accum += adc_result;

	// start another analog to digital conversion
	ADCON0bits.GO = 1;

	i++;

	// check to see if we've got a full sample set
	if(i >= GYRO_SAMPLES_PER_UPDATE)
	{
		// is the completed sample set to replace the current gyro bias?
		if(calc_gyro_bias == 1)
		{
			// convert the accumulator to an integer and update gyro_bias
			gyro_bias = (int)accum;

			// we're done calculating the gyro bias
			calc_gyro_bias = 0;
		}
		else
		{
			// update the gyro rate
			gyro_rate = (int)accum - gyro_bias;

			// integrate the gyro rate to derive the heading 
			gyro_angle += (long)gyro_rate;
		} 

		i = 0;
		accum = 0;
	}	
}
