/*******************************************************************************
*
*	TITLE		gyro.h 
*
*	VERSION:	0.3 (Beta)                           
*
*	DATE:		21-Jan-2005
*
*	AUTHOR:		R. Kevin Watson
*				kevinw@jpl.nasa.gov
*
*	COMMENTS:
*
********************************************************************************
*
*	CHANGE LOG:
*
*	DATE         REV  DESCRIPTION
*	-----------  ---  ----------------------------------------------------------
*	21-Nov-2004  0.1  RKW - Original code.
*	12-Jan-2005  0.2  RKW - Altered Get_Gyro_Rate() and Get_Gyro_Angle() to use
*	                  long integers for internal calculations, allowing larger
*	                  numerators and denominators in the GYRO_RATE_SCALE_FACTOR
*	                  and GYRO_ANGLE_SCALE_FACTOR #defines.
*	12-Jan-2005  0.2  RKW - GYRO_RATE_SCALE_FACTOR and GYRO_ANGLE_SCALE_FACTOR
*	                  #defines added for Analog Devices' ADXRS401, ADXRS150 and
*	                  ADXRS300 gyros.
*	16-Jan-2005  0.3  RKW - Using preprocessor directives, added the ability 
*	                  to select the gyro type, angular units, sample rate and
*	                  number of averaged samples per update.
*	21-Jan-2005  0.3  RKW - Added scaling factors for the BEI GyroChip.
*
*******************************************************************************/

#ifndef _gyro_h
#define _gyro_h

// Pick your gyro by removing the // from one of the six lines below.
// #define GYROCHIP_64	// BEI GyroChip AQRS-00064-xxx
// #define GYROCHIP_75	// BEI GyroChip AQRS-00075-xxx
// #define CRS03		// Silicon Sensing Systems' CRS03-02
// #define ADXRS401 	// Analog Devices' ADXRS401EB
#define ADXRS150		// Analog Devices' ADXRS150EB
// #define ADXRS300		// Analog Devices' ADXRS300EB



// Pick the angular unit by removing the // from one of these two lines.
#define MILLIRADIANS
// #define TENTHS_OF_A_DEGREE


// For optimum performance, you'll need to calibrate the scaling factor 
// to match that of your gyro's. One way to calibrate your gyro is to 
// mount it very securely to a hefty, square or rectangular object. 
// Mounting the gyro to a hefty object will help dampen higher frequency 
// vibrations that can adversly effect your measurements. Place the 
// mounted gyro against another square object and start the included 
// demonstration software. To get good results, the mount must be 
// absolutely still when the "Calibrating Gyro Bias..." message appears 
// on the terminal screen. After a few seconds, the gyro angular rate 
// and angle will be sent to the terminal screen. If the angle drifts 
// rapidly while the mounted gyro is motonless, you need to restart the 
// software to acquire a new gyro bias measurement. Again, gyros are 
// very sensitive and must be still while the bias is calculated. Once 
// the gyro is running with little drift, rotate the mount 180 degrees 
// and note the reported angle. If the angular units are set to tenths 
// of a degree, the ideal reported angle is 1800. If set to milliradians, 
// the ideal angle 1s 3142 (Pi times a thousand). For every tenth of a 
// percent that the angle is high, decrease the GYRO_CAL_FACTOR numerator 
// by one. Conversly, for every tenth of a percent low, increase the 
// numerator by one. Repeat until you're satisfied with the accuracy.
#define GYRO_CAL_FACTOR 1000/1000


// Pick a gyro sample rate by uncommenting one of these lines. Faster 
// is better, but your CPU will spend more time servicing interrupts. 
// Pick the slowest sample rate that still meets your performace criteria.
// These #defines are used below to set the value of GYRO_SAMPLE_RATE and
// to set the timer 2 update rate in gyro.c/Initialize_Timer_2().
// #define GYRO_SAMPLE_RATE_200HZ
#define GYRO_SAMPLE_RATE_400HZ
// #define GYRO_SAMPLE_RATE_800HZ
// #define GYRO_SAMPLE_RATE_1600HZ


// Number of ADC samples that will be averaged for each gyro update. More 
// is better, but your update rate will decrease proportionatly. This must 
// be sixty-four or less. The gyro sample rate divided by number of samples
// per update should be at least equil to fifty, which is the number of
// gyro updates per second.
#define GYRO_SAMPLES_PER_UPDATE 8

// If you modify stuff below this line, you'll break the software.

#define ADC_VOLTS_PER_BIT 5/1024
#define MILLIRADIANS_PER_DEGREE 349/20
#define TENTHS_OF_A_DEGREE_PER_DEGREE 10

#ifdef GYRO_SAMPLE_RATE_200HZ
#define GYRO_SAMPLE_RATE 200
#endif

#ifdef GYRO_SAMPLE_RATE_400HZ
#define GYRO_SAMPLE_RATE 400
#endif

#ifdef GYRO_SAMPLE_RATE_800HZ
#define GYRO_SAMPLE_RATE 800
#endif

#ifdef GYRO_SAMPLE_RATE_1600HZ
#define GYRO_SAMPLE_RATE 1600
#endif

// BEI GyroChip AQRS-00064-xxx (sensitivity = 35.0mV/deg/sec)
#ifdef GYROCHIP_64
#define GYRO_SENSITIVITY 28	// in units of deg/sec/volt
#endif

// BEI GyroChip AQRS-00075-xxx (sensitivity = 30.0mV/deg/sec)
#ifdef GYROCHIP_75
#define GYRO_SENSITIVITY 33	// in units of deg/sec/volt
#endif

// Silicon Sensing Systems' CRS03 (sensitivity = 20mV/deg/sec)
#ifdef CRS03
#define GYRO_SENSITIVITY 50		// in units of deg/sec/volt
#endif

// Analog Devices' ADXRS401 (sensitivity = 15mV/deg/sec)
#ifdef ADXRS401
#define GYRO_SENSITIVITY 67		// in units of deg/sec/volt
#endif

// Analog Devices' ADXRS150 (sensitivity = 12.5mV/deg/sec)
#ifdef ADXRS150
#define GYRO_SENSITIVITY 80		// in units of deg/sec/volt
#endif

// Analog Devices' ADXRS300 (sensitivity = 5.0mV/deg/sec) 
#ifdef ADXRS300
#define GYRO_SENSITIVITY 200	// in units of deg/sec/volt
#endif

#ifdef MILLIRADIANS
#define GYRO_RATE_SCALE_FACTOR GYRO_SENSITIVITY * ADC_VOLTS_PER_BIT * MILLIRADIANS_PER_DEGREE
#endif

#ifdef TENTHS_OF_A_DEGREE
#define GYRO_RATE_SCALE_FACTOR GYRO_SENSITIVITY * ADC_VOLTS_PER_BIT * TENTHS_OF_A_DEGREE_PER_DEGREE
#endif


void Initialize_Gyro(void);		// initializes and starts the gyro software
int Get_Gyro_Rate(void);		// returns the current heading angular rate of change
long Get_Gyro_Angle(void);		// returns the current heading angle
void Calc_Gyro_Bias(void);		// calculates the current gyro bias
int Get_Gyro_Bias(void);		// returns the current calculated gyro bias
void Set_Gyro_Bias(int);		// manually sets the gyro bias
void Reset_Gyro_Angle(void);	// resets the heading angle to zero
void Disable_Gyro(void);		// disables the gyro
void Initialize_ADC(void);		// initializes the analog to digital converter
void Initialize_Timer_2(void);	// initializes and starts timer 2
void Timer_2_Int_Handler(void);	// interrupt service routine for timer 2

#endif
