/*******************************************************************************
* FILE NAME: robot.c
*
* DESCRIPTION: 
*  This is a set of robot control algorithms  
*
* USAGE:
*  Place the robot_init() function in User_Initialization and
*  replace the default function in user_routines.c with the 
*  function robot_control()
*******************************************************************************/

#include "ifi_aliases.h"
#include "ifi_default.h"
#include "ifi_utilities.h"
#include "printf_lib.h"
#include "robot.h"
#include "commands.h"

//#define DEBUG      // Uncomment this line to get helpful debug messages 

/* Global Variables */

int current_command = 0;
long unsigned int robot_time = 0;   /* millisec */
int pos_x = 0;                      /* mm */
int pos_y = 0;                      /* mm */
long int heading = 0;               /* milliradians */

/* Functions */

void robot_init(void)
{
/* Set our sensors as inputs */
rc_dig_in01 = rc_dig_in03 = rc_dig_in04 = rc_dig_in05 = INPUT; 

/* Initialize PID Control */
pid_init();

/* Set starting state */

pos_x = 0;
pos_y = 0;
heading = PI_MRAD / 2;

}

void robot_control(void)
{
robot_command();
robot_timer();
robot_position();
}

void robot_command(void)
{
int rc;             /* Return code from commands */

/* These three lines need to execute every cycle */
pid_time++;
pid(LEFT);
pid(RIGHT);


switch (command_list[current_command].command)
  {
  case NULL:
    {
	rc = 0;    // We don't want to increment to the next command
    break;
    }

  case CMD_SHOW_STATE:
    {
    rc = cmd_show_state();
    break;
    }

  case CMD_VELOCITY:
    {
    rc = cmd_velocity();
    break;
    }

  case CMD_WAIT:
    {
    rc = cmd_wait();
    break;
    }

  case CMD_WAIT_UNTIL:
    {
    rc = cmd_wait_until();
    break;
    }

  case CMD_DRIVE:
    {
    rc = cmd_drive();
    break;
    }

  case CMD_TURN:
    {
    rc = cmd_turn();
    break;
    }

  case CMD_GOTO_WAYPOINT:
    {
    rc = cmd_goto_waypoint();
    break;
    }

  case CMD_SET_POS:
    {
    rc = cmd_set_pos();
    break;
    }

  case CMD_SET_HEADING:
    {
    rc = cmd_set_heading();
    break;
    }

  case CMD_WAIT_FOR_BUMP:
    {
    rc = cmd_wait_for_bump();
    break;
    }

  case CMD_STOP:
    {
    rc = cmd_stop();
    break;
    }

  case CMD_JUMP:
    {
    rc = cmd_jump();
    break;
    }

  case CMD_GYRO_BIAS:
    {
    rc = cmd_gyro_bias();
    break;
    }

  case CMD_KEEP_HEADING:
    {
    rc = cmd_keep_heading();
    break;
    }

  default:
    {
    printf("Unknown command, restarting command list\n");
    current_command = 0;  //Just reset to first command
    break;
    }
  }

#ifdef DEBUG
printf ("current_command  = %d\n", (int)current_command);
printf ("command_list[current_command].command  = %d\n", 
              (int)command_list[current_command].command);
printf ("command_list[current_command].parm_1  = %d\n", 
              (int)command_list[current_command].parm_1);
printf ("rc  = %d\n", (int)rc);

#endif


if (rc)
  {
  /* If the command is finished, go to the next */
  current_command++;
  }

}

void robot_timer(void)
{
robot_time += 26;  /* add 26 msec per cycle */
}

void robot_position(void)
{
static long int old_pos = 0;
long int new_pos;
long int dist;

new_pos = (get_pos(LEFT) + get_pos(RIGHT)) / 2L;
dist = ((new_pos - old_pos)*(WHEEL_RADIUS * TWOPI_MRAD))/(COUNTS_PER_REV * 1000L);
old_pos = new_pos;
heading = Get_Gyro_Angle();
//pos_x += sin_int(heading) * dist;
//pos_y += cos_int(heading) * dist;

#ifdef DEBUG
if (!(pid_time % 2))
  {
  printf("dist = %d , heading = %d\n", (int)dist, (int)heading);
  } 
#endif

}


/***********************************************************************
      Robot Command Definitions 
************************************************************************/



int cmd_show_state(void)
{
int rc = 0;

printf("robot_time (msec) = %d\n", robot_time );
printf("pos_x (mm) = %d\n", pos_x);
printf("pos_y (mm) = %d\n", pos_y);
printf("heading (milliradians) = %d\n", heading);

rc = DONE;

return rc;
}

/***********************************************************************
       Set velocity in mm/s
************************************************************************/

int cmd_velocity(void)
{
int rc = 0;

long int vel = command_list[current_command].parm_1;
long int mrad_per_sec;
long int counts_per_sec;
long int mm_per_sec;

/* Get the velocity, but don't let it be more than the max */
if (mm_per_sec > MAX_VELOCITY_MM_PER_SEC) 
	mm_per_sec = MAX_VELOCITY_MM_PER_SEC;
else if (mm_per_sec < -MAX_VELOCITY_MM_PER_SEC) 
	mm_per_sec = -MAX_VELOCITY_MM_PER_SEC;
else
	mm_per_sec = vel;

mrad_per_sec = (mm_per_sec * 1000L) / WHEEL_RADIUS;
counts_per_sec = (mrad_per_sec * COUNTS_PER_REV) / TWOPI_MRAD;

printf("mm_per_sec = %d   mrad_per_sec = %d   counts_per_sec = %d\n", 
	(int)mm_per_sec,
	(int)mrad_per_sec, (int)counts_per_sec);


set_vel(LEFT,  counts_per_sec);
set_vel(RIGHT, counts_per_sec);	

rc = 1;
return rc;
}

int cmd_wait(void)
{
static int state = START;
static long int end_time;
int rc = 0;

switch (state)
  {
  case START:
    {
    end_time = robot_time + command_list[current_command].parm_1;
	state = IN_PROGRESS;
    rc = 0;
    break;
    }
  case IN_PROGRESS:
    {
    if (robot_time > end_time)
      {
      state = COMPLETE;
      }
    rc = 0;
    break;
    }
  case COMPLETE:
    {
    printf("Done CMD_WAIT\n");
	state = START;
    rc = 1;
    break;
    }
  }
return rc;
}

int cmd_wait_until(void)
{
static int state = START;
static long int end_time;
int rc = 0;

switch (state)
  {
  case START:
    {
    end_time = command_list[current_command].parm_1;
	state = IN_PROGRESS;
    rc = 0;
    break;
    }
  case IN_PROGRESS:
    {
    if (robot_time > end_time)
      {
      state = COMPLETE;
      }
    rc = 0;
    break;
    }
  case COMPLETE:
    {
    printf("Done CMD_WAIT_UNTIL\n");
	state = START;
    rc = 1;
    break;
    }
  }
return rc;
}


int cmd_drive(void)
{
static int state = START;
static long int counts;
int rc = 0;
long int mm = (long int)command_list[current_command].parm_1;

switch (state)
  {
  case START:
    {
	counts = (mm * (COUNTS_PER_REV * 1000L))/(WHEEL_RADIUS * TWOPI_MRAD);
    printf("%d mm = %d encoder counts\n", (int)mm, (int)counts);
	state = IN_PROGRESS;
    rc = 0;
    break;
    }
  case IN_PROGRESS:
    {
    set_pid_stop(LEFT);
    set_pid_stop(RIGHT);
	state = COMPLETE;
    rc = 0;
    break;
    }
  case COMPLETE:
    {
    set_pos(LEFT, counts);
    set_pos(RIGHT, counts);
	state = START;
    rc = 1;
    break;
    }
  }
return rc;
}


int cmd_turn (void)
{
static int state = START;
static long int goal_angle;
static long int tolerance;
int rc = 0;
long int heading_error;

switch (state)
  {
  case START:
    {
    goal_angle = heading + command_list[current_command].parm_1;
    tolerance = command_list[current_command].parm_2;
    printf("Turning to heading %d\n", (int)goal_angle);
	state = TURNING;
    rc = 0;
    break;
    }
  case TURNING:
  case STOPPED:
    {
    heading_error = goal_angle - heading;
    if (heading_error < -tolerance)
	  {
	  // Turn Left
      set_vel(LEFT, TURN_SPEED);
      set_vel(RIGHT, -TURN_SPEED);      
      state = TURNING;
      }
    else if (heading_error > tolerance)
	  {
	  // Turn Right
      set_vel(LEFT,  -TURN_SPEED);
      set_vel(RIGHT, TURN_SPEED);      
      state = TURNING;
      }
    else 
      {
      set_pid_stop(LEFT);
      set_pid_stop(RIGHT);
      state = COMPLETE;
      }
    rc = 0;
    break;
    }
  case COMPLETE:
    {
	state = START;
    rc = 1;
    break;
    }
  }


/* Use this to turn without a gyro
static int state = START;
static long int counts;
int rc = 0;
long int mm;
long int goal_angle = command_list[current_command].parm_1;

switch (state)
  {
  case START:
    {
    mm = (goal_angle * WHEEL_BASE) / 1000L;
	counts = (mm * (COUNTS_PER_REV * 1000L))/(WHEEL_RADIUS * TWOPI_MRAD);
    if (counts < 0) 
		dir = -1; 
	else
		dir = 1;
    printf("%d mm = %d encoder counts,  %d = dir\n", (int)mm, (int)counts,(int)dir);
    set_pid_stop(LEFT);
    set_pid_stop(RIGHT);
	state = IN_PROGRESS;
    rc = 0;
    break;
    }
  case IN_PROGRESS:
    {
    set_vel(LEFT, -(TURN_SPEED * dir));
    set_vel(RIGHT, (TURN_SPEED * dir));
	state = COMPLETE;
    rc = 0;
    break;
    }
  case COMPLETE:
    {
	if ((dir * get_pos(RIGHT)) > (dir * counts))
     {
      set_pid_stop(LEFT);
      set_pid_stop(RIGHT);
	  state = START;
      rc = 1;
      }
    break;
    }
  }

*/

return rc;
}


int cmd_goto_waypoint(void)
{

return DONE;
}


int cmd_set_pos(void)
{
static int state = START;
int rc = 0;
/*
switch (state)
  {
  case START:
    {
    printf("Hi 1\n");
	state = IN_PROGRESS;
    rc = 0;
    break;
    }
  case IN_PROGRESS:
    {
    printf("Hi 2\n");
	state = COMPLETE;
    rc = 0;
    break;
    }
  case COMPLETE:
    {
    printf("Hi 3\n");
	state = START;
    rc = 1;
    break;
    }
  }
*/
return rc;
}


int cmd_set_heading(void)
{
static int state = START;
int rc = 0;
/*
switch (state)
  {
  case START:
    {
    printf("Hi 1\n");
	state = IN_PROGRESS;
    rc = 0;
    break;
    }
  case IN_PROGRESS:
    {
    printf("Hi 2\n");
	state = COMPLETE;
    rc = 0;
    break;
    }
  case COMPLETE:
    {
    printf("Hi 3\n");
	state = START;
    rc = 1;
    break;
    }
  }
*/
return rc;
}


int cmd_wait_for_bump(void)
{
static int state = START;
static long int start_time;
int rc = 0;

switch (state)
  {
  case START:
    {
    printf("Waiting for bump switch to be hit\n");
	state = IN_PROGRESS;
    rc = 0;
    break;
    }
  case IN_PROGRESS:
    {
    if ((bump_f == 0) || (p1_sw_trig == 1))
	  {
	  state = COMPLETE;
      }
    rc = 0;
    break;
    }
  case COMPLETE:
    {
	state = START;
    rc = 1;
    break;
    }
  }
return rc;
}

int cmd_stop(void)
{
    set_pid_stop(LEFT);
    set_pid_stop(RIGHT);

return DONE;
}

int cmd_jump(void)
{
int rc = 0;

current_command = command_list[current_command].parm_1;

return rc;
}

int cmd_gyro_bias(void)
{
Calc_Gyro_Bias();
return DONE;
}

int cmd_keep_heading(void)
{
static int state = START;
static long int start_heading;
static long int tolerance;
static long int time_out;

int rc = 0;
long int heading_error;

switch (state)
  {
  case START:
    {
    start_heading = heading;
    time_out = robot_time + command_list[current_command].parm_1;
    tolerance = command_list[current_command].parm_2;
    printf("Maintaining heading %d\n", start_heading);
	state = TURNING;
    rc = 0;
    break;
    }
  case TURNING:
  case STOPPED:
    {
    heading_error = start_heading - heading;
    // Whether we are turning or stopped we want to always check heading
    if (heading_error < -tolerance)
	  {
	  // Turn Left
      set_vel(LEFT, TURN_SPEED);
      set_vel(RIGHT, -TURN_SPEED);      
      state = TURNING;
      }
    else if (heading_error > tolerance)
	  {
	  // Turn Right
      set_vel(LEFT,  -TURN_SPEED);
      set_vel(RIGHT, TURN_SPEED);      
      state = TURNING;
      }
    else 
      {
      if (state == TURNING)
        {
        set_pid_stop(LEFT);
        set_pid_stop(RIGHT);
        state = STOPPED;
        }
      else
        {
        // We are stopped, so maintain postion
        set_pos(LEFT, 0);
        set_pos(RIGHT, 0);
        }
      }

	if (robot_time > time_out)
      {
      state = COMPLETE;
      }

    rc = 0;
    break;
    }
  case COMPLETE:
    {
	state = START;
    rc = 1;
    break;
    }
  }
return rc;
}

 /******************************************************************************/
