/*******************************************************************************
* FILE NAME: robot.h
*
* DESCRIPTION: 
*  Header file for robot.c
*  This is a set of functions for commanding and controlling a wheeled
*  robot.  During each control cycle we will process the current command
*  from a command list, update the vehicle clock, and update the position.
*  In order to use integer functions as much as possible, we will use units
*  of millimeters, milliradians and milliseconds.  That will give us
*  good resolution for all our calculations.
*
* USAGE:
*  Include in user_routines.c
*  
*  To simplify calculations, we are assuming a 4 wheeled tractor steered
*  vehicle.  The center of rotation will be about the center of the wheel
*  pair on each side.  We will do the calculations as if we had a two 
*  wheeled vehicle, and in fact this code could operate a four wheel drive
*  vehicle just as well.
*******************************************************************************/

#ifndef __robot_h_
#define __robot_h_

  /************/
 /* Includes */
/************/

/*
We use a gyro, encoders, and pid control to run the robot.  By
including the header files here you don't need to include them
everywhere.  Just add robot.h and the others get included automatically.
*/

#include "gyro.h"
#include "encoder.h"
#include "pid.h"


  /***********/
 /* Structs */
/***********/

/* 
  This is the structure that is used for each command.  Each command 
  has up to three parameters associated with it.  For large numbers
  (greater than 32K), the first parameter is an long integer.  The
  other two are regular integers.   

  Look at the command list in commands.h to see examples of commands
  that you can use on your robot.  If you do not want to add new commands,
  you should only have to edit the commands.h file.
*/

struct commands
{
int command;
long int parm_1;
int parm_2;
int parm_3;
};

  /**********************/
 /* Command Dictionary */
/**********************/

/*
The command dictionary is a list of commands that can be put in a 
sequence to make it easy to program the autonomous motion of the robot.
Each command has a define statement and a function prototype associated
with it.  The define value is used in the command list to identify
the commands.  The function is defined in robot.c  

If you want to add your own commands, create a define with a new number
and list the function prototype here.  Then go to robot.c to write
your code.

Each command has 0 to 3 parameters.  Just fill in zeros for unused 
parameters in the command list. 
*/

/* The NULL command is used to end the command list.  You must
   have a NULL command as the last command in your command list
   so that the command processor knows when to stop. */  
#define NULL                  0

/* CMD_SHOW_STATE will display the current position and heading
   of the robot to the programming host.  It is useful for debugging
   your driving. It takes no parameters */
#define CMD_SHOW_STATE        1
int cmd_show_state(void);

/* CMD_VELOCITY takes one parameter, the velocity in mm/sec.  The 
   robot will remain at this velocity until you give it a CMD_STOP. */
#define CMD_VELOCITY          2
int cmd_velocity(void);

/* CMD_WAIT takes one parameter, the time to wait in milliseconds.  It
   will not let the next command start until that much time has elapsed.
   You might want to use it after a CMD_VELOCITY or CMD_DRIVE to let
   them finish before going on. */
#define CMD_WAIT              3
int cmd_wait(void);

/* CMD_WAIT_UNTIL takes one parameter, the time from the start of the
   match.  The robot will not go on to the next command until that
   much time has elapsed.  This might be useful if you want your robot
   to sit still until 5 seconds into the match (to let the other robots
   get out of the way???). */
#define CMD_WAIT_UNTIL        4
int cmd_wait_until(void);

/* CMD_DRIVE takes one parameter, a distance in mm.  The robot will
   drive straight that distance and try to maintain the position. */
#define CMD_DRIVE             5
int cmd_drive(void);

/* CMD_TURN takes two parameters, an angle to turn through and a
   tolerance, both in milliradians.  */
#define CMD_TURN              6
int cmd_turn (void);

/* CMD_GOTO_WAYPOINT is not yet implemented (maybe when we get trig
   functions ;-) */
#define CMD_GOTO_WAYPOINT     7
int cmd_goto_waypoint(void);

/* CMD_SET_POS is not yet implemented (maybe when we get trig
   functions ;-) */
#define CMD_SET_POS           8
int cmd_set_pos(void);

/* CMD_SET_HEADING is not yet implemented (maybe when we get trig
   functions ;-) */
#define CMD_SET_HEADING       9
int cmd_set_heading(void);

/* CMD_WAIT_FOR_BUMP takes no parameters.  It will wait until it sees
   a signal from the bump sensor or the trigger to continue to the next 
   command.  This is useful for debugging by waiting between commands
   until you hit the bump sensor, or the trigger.  It is also useful 
   for driving until you hit something.  */
#define CMD_WAIT_FOR_BUMP     10
int cmd_wait_for_bump(void);

/* CMD_STOP takes no parameters. It just stops driving.  */
#define CMD_STOP              11
int cmd_stop(void);

/* CMD_JUMP takes one parameter a command list position.  It is used
   to jump around in the command list.  Note: the command list starts
   with command number 0.  This is C after all. */
#define CMD_JUMP              12
int cmd_jump(void);

/* CMD_GYRO_BIAS sets the gyro bias.  It takes no parameters.  Run this
   while stopped before using the gyro.  It should calibrate out any
   drift.  */
#define CMD_GYRO_BIAS         13
int cmd_gyro_bias(void);

/* CMD_KEEP_HEADING takes two parameters, a timeout and a tolerance.  It
   is a demonstration of the robot maintaining position and orientation.
   While this command is active, the robot will fight against any attempt
   to move it.  */
#define CMD_KEEP_HEADING      14
int cmd_keep_heading(void);

/* Command States */
/* These states are used by the commands to keep track of the robot's 
   progress in completing each command. */
#define START              1
#define IN_PROGRESS        2
#define COMPLETE           3
#define TURNING            4
#define DRIVING            5
#define STOPPED            6

/* Return values */
/* Commands return working to tell the command processor not to
   move on to the next command yet.  DONE indicates it is OK to
   go to the next command. */
#define WORKING   0
#define DONE      1


  /***********************/
 /* Function prototypes */
/***********************/

/*
  The following two functions are your interface to the 
  standard FRC code.  Put robot_init() in the User_Initialization()
  function in user_routines.c.  Put robot_control() in the 
  User_Autonomous_Code() function in user_routines_fast.c or in
  the Process_Data_From_Master_uP() function in user_routines.c.
*/
void robot_init(void);       // This function is called one at startup
void robot_control(void);    // This function is called every 26.2 msec

/* functions called from robot_control() */
void robot_command(void);
void robot_position(void);
void robot_timer(void);

  /***********/
 /* Defines */
/***********/

/*
  Some useful math constants.  (Remember we are using milliradians)
*/
#define PI_MRAD 3142L
#define TWOPI_MRAD 6283L

/* 
  If you want to limit the max velocity of your robot set it here
*/  
#define MAX_VELOCITY_MM_PER_SEC 5000

/* Vehicle constants */

/* This is the diameter of the kit wheels */
#define WHEEL_DIAMETER 220L  
#define WHEEL_RADIUS (WHEEL_DIAMETER/2)   
/* This is the effective wheel base of the robot
   measured side to side between wheels */
#define WHEEL_BASE 800L
/* We need the number of encoder counts for each wheel revolution
   the first line is the number of counts for one encoder revolution
   we can use the gear ratios to figure the rest out.  */
#define ENCODER_COUNTS_PER_REV 128L
#define ENCODER_GEAR 21
#define WHEEL_GEAR 28
#define COUNTS_PER_REV (ENCODER_COUNTS_PER_REV * WHEEL_GEAR) / ENCODER_GEAR

/* Sensor Definitions */
/* The front bumper is plugged into dio 3.  I like to use
   the bumper to step through code by having a command that
   waits until the bumper is touched. */
#define bump_f   rc_dig_in03

/* How fast do we want the robot to go while turning? */
#define TURN_SPEED 1500L

  /********************/
 /* Global Variables */
/********************/

/* 
These are global variables that are used by the command processor. */
extern int current_command;            /* keeps track of the current command */
extern long unsigned int robot_time;   /* msec - The time since the match started */
extern int pos_x, pos_y;               /* mm - where is the robot now */
extern long int heading;               /* milliradians - and where is it facing */
extern struct commands command_list[]; /* list of commands - see commands.h*/

#endif /* __robot_h */
